import pytest
from unittest.mock import AsyncMock, patch, Mock
from medhaai import AdvancedWebScraperTool, ToolKit
from medhaai.exceptions import WebScrapingError

@pytest.fixture
def mock_response():
    mock = AsyncMock()
    mock.__aenter__.return_value.status = 200
    mock.__aenter__.return_value.text.return_value = "<html><body><p>Test content</p></body></html>"
    return mock

@pytest.mark.asyncio
async def test_web_scraper(mock_response):
    with patch('aiohttp.ClientSession.get', return_value=mock_response):
        with patch('newspaper.Article') as mock_article:
            mock_article.return_value.title = "Test Title"
            mock_article.return_value.text = "Test content"
            mock_article.return_value.summary = "Test summary"
            mock_article.return_value.keywords = ["test", "keywords"]

            scraper = AdvancedWebScraperTool()
            result = await scraper.scrape("http://example.com")

            assert result['title'] == "Test Title"
            assert result['text'] == "Test content"
            assert result['summary'] == "Test summary"
            assert result['keywords'] == ["test", "keywords"]

@pytest.mark.asyncio
async def test_web_scraper_error():
    mock_response = AsyncMock()
    mock_response.__aenter__.return_value.status = 404
    
    with patch('aiohttp.ClientSession.get', return_value=mock_response):
        scraper = AdvancedWebScraperTool()
        with pytest.raises(WebScrapingError):
            await scraper.scrape("http://example.com")

@pytest.mark.asyncio
async def test_toolkit():
    toolkit = ToolKit()
    mock_result = {"result": "test"}
    with patch.object(AdvancedWebScraperTool, 'scrape', new_callable=AsyncMock, return_value=mock_result):
        result = await toolkit.use_tool("web_scraper", url="http://example.com")
        assert result == mock_result

    with pytest.raises(WebScrapingError):
        await toolkit.use_tool("non_existent_tool")


class MedhaAIException(Exception):
    """Base exception for MedhaAI"""

class LLMError(MedhaAIException):
    """Raised when there's an error with LLM operations"""

class ToolError(MedhaAIException):
    """Raised when there's an error with tool operations"""

class ConfigError(MedhaAIException):
    """Raised when there's a configuration error"""

class WebScrapingError(ToolError):
    """Raised when there's an error during web scraping"""

class UnsupportedModelError(MedhaAIException):
    """Raised when an unsupported model is requested"""

class RateLimitError(MedhaAIException):
    """Raised when rate limit is exceeded"""


from pydantic_settings import BaseSettings, SettingsConfigDict
from pydantic import Field
from typing import Optional

class MedhaConfig(BaseSettings):
    openai_api_key: Optional[str] = Field(None, env='OPENAI_API_KEY')
    anthropic_api_key: Optional[str] = Field(None, env='ANTHROPIC_API_KEY')
    llm_model_name: str = Field('gpt-3.5-turbo', env='MEDHA_MODEL_NAME')
    max_tokens: int = Field(500, env='MEDHA_MAX_TOKENS')
    temperature: float = Field(0.7, env='MEDHA_TEMPERATURE')
    request_timeout: int = Field(30, env='MEDHA_REQUEST_TIMEOUT')
    max_retries: int = Field(3, env='MEDHA_MAX_RETRIES')

    model_config = SettingsConfigDict(env_file='.env', env_file_encoding='utf-8', case_sensitive=False)


from typing import Dict, Any
import aiohttp
from bs4 import BeautifulSoup
from newspaper import Article
import nltk
from .exceptions import WebScrapingError
from .logging import setup_logger

logger = setup_logger(__name__)

nltk.download('punkt', quiet=True)

class AdvancedWebScraperTool:
    async def scrape(self, url: str) -> Dict[str, Any]:
        try:
            async with aiohttp.ClientSession() as session:
                async with session.get(url, timeout=30) as response:
                    if response.status == 200:
                        html = await response.text()
                        return await self._process_html(html, url)
                    else:
                        logger.error(f"Failed to fetch URL: {url}. Status code: {response.status}")
                        raise WebScrapingError(f"Failed to fetch URL: {url}. Status code: {response.status}")
        except aiohttp.ClientError as e:
            logger.error(f"Network error during web scraping: {str(e)}")
            raise WebScrapingError(f"Network error during web scraping: {str(e)}")
        except Exception as e:
            logger.error(f"Error during web scraping: {str(e)}")
            raise WebScrapingError(f"Error during web scraping: {str(e)}")

    async def _process_html(self, html: str, url: str) -> Dict[str, Any]:
        try:
            article = Article(url)
            article.set_html(html)
            article.parse()
            article.nlp()

            soup = BeautifulSoup(html, 'html.parser')
            links = [a['href'] for a in soup.find_all('a', href=True) if a['href'].startswith('http')]

            return {
                'title': article.title,
                'text': article.text[:1000],
                'summary': article.summary,
                'keywords': article.keywords,
                'links': links[:10]
            }
        except Exception as e:
            logger.error(f"Error processing HTML: {str(e)}")
            raise WebScrapingError(f"Error processing HTML: {str(e)}")

class ToolKit:
    def __init__(self):
        self.tools: Dict[str, Any] = {
            "web_scraper": AdvancedWebScraperTool().scrape
        }

    async def use_tool(self, tool_name: str, **kwargs) -> Any:
        if tool_name not in self.tools:
            logger.error(f"Tool {tool_name} not found")
            raise WebScrapingError(f"Tool {tool_name} not found")
        return await self.tools[tool_name](**kwargs)


import pytest
from medhaai import MedhaConfig

def test_config_validation():
    config = MedhaConfig(openai_api_key="test_key", llm_model_name="gpt-3.5-turbo")
    assert config.llm_model_name == "gpt-3.5-turbo"
    assert config.max_tokens == 500
    assert config.temperature == 0.7

def test_config_env_variables(monkeypatch):
    monkeypatch.setenv("OPENAI_API_KEY", "test_env_key")
    monkeypatch.setenv("MEDHA_MODEL_NAME", "gpt-4")
    monkeypatch.setenv("MEDHA_MAX_TOKENS", "1000")
    
    config = MedhaConfig()
    assert config.openai_api_key == "test_env_key"
    assert config.llm_model_name == "gpt-4"
    assert config.max_tokens == 1000


from typing import Dict, Any
from .llm import MedhaLLM
from .tools import ToolKit
from .exceptions import MedhaAIException
from .logging import setup_logger

logger = setup_logger(__name__)

class MedhaAgent:
    def __init__(self, llm: MedhaLLM, toolkit: ToolKit):
        self.llm = llm
        self.toolkit = toolkit

    async def run(self, task: str) -> str:
        try:
            logger.info(f"Starting task: {task}")
            
            # Generate a plan
            plan = await self.generate_plan(task)
            logger.info(f"Generated plan: {plan}")
            
            # Execute the plan
            result = await self.execute_plan(plan)
            logger.info("Plan execution completed")
            
            # Analyze and summarize the result
            summary = await self.analyze_and_summarize(result)
            logger.info("Task completed successfully")
            
            return summary
        except Exception as e:
            logger.error(f"Error in agent execution: {str(e)}")
            raise MedhaAIException(f"Error in agent execution: {str(e)}")

    async def generate_plan(self, task: str) -> str:
        prompt = f"Given the task: '{task}', create a step-by-step plan to accomplish it. If needed, use the web_scraper tool to gather information. Provide the plan in a numbered list format."
        return await self.llm.generate(prompt)

    async def execute_plan(self, plan: str) -> Dict[str, Any]:
        steps = plan.split('\n')
        results = {}
        for step in steps:
            logger.info(f"Executing step: {step}")
            if "web_scraper" in step.lower():
                url = step.split("web_scraper:")[-1].strip()
                results[step] = await self.toolkit.use_tool("web_scraper", url=url)
            else:
                results[step] = await self.llm.generate(step)
        return results

    async def analyze_and_summarize(self, results: Dict[str, Any]) -> str:
        prompt = "Analyze and summarize the following information:\n\n"
        for step, result in results.items():
            prompt += f"Step: {step}\nResult: {result}\n\n"
        prompt += "Provide a concise summary of the key findings and insights."
        return await self.llm.generate(prompt)


import pytest
from unittest.mock import AsyncMock, patch, Mock
from medhaai import MedhaConfig, MedhaLLM
from medhaai.exceptions import UnsupportedModelError, LLMError
from openai import RateLimitError, APIError

@pytest.fixture
def config():
    return MedhaConfig(openai_api_key="test_key", llm_model_name="gpt-3.5-turbo")

@pytest.mark.asyncio
async def test_llm_generation(config):
    with patch('openai.AsyncOpenAI') as mock_openai:
        mock_completion = Mock()
        mock_completion.choices = [Mock(message=Mock(content="Test response"))]
        mock_openai.return_value.chat.completions.create = AsyncMock(return_value=mock_completion)
        llm = MedhaLLM(config)
        response = await llm.generate("Test prompt")
        assert response == "Test response"

@pytest.mark.asyncio
async def test_unsupported_model():
    with pytest.raises(UnsupportedModelError):
        config = MedhaConfig(openai_api_key="test_key", llm_model_name="unsupported-model")
        MedhaLLM(config)

@pytest.mark.asyncio
async def test_rate_limit_retry(config):
    with patch('openai.AsyncOpenAI') as mock_openai:
        mock_completion = Mock()
        mock_completion.choices = [Mock(message=Mock(content="Test response"))]
        mock_openai.return_value.chat.completions.create = AsyncMock(side_effect=[
            RateLimitError("Rate limit exceeded", response=Mock(status_code=429)),
            mock_completion
        ])
        llm = MedhaLLM(config)
        response = await llm.generate("Test prompt")
        assert response == "Test response"

@pytest.mark.asyncio
async def test_max_retries_exceeded(config):
    with patch('openai.AsyncOpenAI') as mock_openai:
        mock_openai.return_value.chat.completions.create = AsyncMock(side_effect=APIError("API Error"))
        llm = MedhaLLM(config)
        with pytest.raises(LLMError):
            await llm.generate("Test prompt")


from .agent import MedhaAgent
from .llm import MedhaLLM
from .tools import AdvancedWebScraperTool, ToolKit
from .config import MedhaConfig
from .logging import setup_logger

__all__ = ['MedhaAgent', 'MedhaLLM', 'AdvancedWebScraperTool', 'ToolKit', 'MedhaConfig', 'setup_logger']
__version__ = '0.3.0'

# Setup default logger
setup_logger()


import pytest
from unittest.mock import AsyncMock, patch
from medhaai import MedhaAgent, MedhaLLM, ToolKit, MedhaConfig

@pytest.fixture
def mock_llm():
    return AsyncMock(spec=MedhaLLM)

@pytest.fixture
def mock_toolkit():
    return AsyncMock(spec=ToolKit)

@pytest.fixture
def agent(mock_llm, mock_toolkit):
    return MedhaAgent(mock_llm, mock_toolkit)

@pytest.mark.asyncio
async def test_agent_run(agent, mock_llm, mock_toolkit):
    mock_llm.generate.side_effect = [
        "1. Step 1\n2. web_scraper: http://example.com\n3. Step 3",
        "Result 1",
        "Result 3",
        "Final summary"
    ]
    mock_toolkit.use_tool.return_value = {"result": "Scraped data"}

    result = await agent.run("Test task")

    assert result == "Final summary"
    assert mock_llm.generate.call_count == 4
    assert mock_toolkit.use_tool.call_count == 1

@pytest.mark.asyncio
async def test_agent_error_handling(agent, mock_llm):
    mock_llm.generate.side_effect = Exception("Test error")

    with pytest.raises(Exception):
        await agent.run("Test task")

@pytest.mark.asyncio
async def test_agent_context_update(agent):
    await agent.update_context({"key": "value"})
    assert agent.context == {"key": "value"}


import logging
import sys

def setup_logger(name='medhaai', level=logging.INFO):
    logger = logging.getLogger(name)
    logger.setLevel(level)

    if not logger.handlers:
        handler = logging.StreamHandler(sys.stdout)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        handler.setFormatter(formatter)
        logger.addHandler(handler)

    return logger


import asyncio
from openai import AsyncOpenAI, RateLimitError
from anthropic import AsyncAnthropic
from .config import MedhaConfig
from .exceptions import LLMError, UnsupportedModelError
from .logging import setup_logger

logger = setup_logger(__name__)

class MedhaLLM:
    def __init__(self, config: MedhaConfig):
        self.config = config
        if 'gpt' in config.llm_model_name:
            self.client = AsyncOpenAI(api_key=config.openai_api_key)
            self.provider = 'openai'
        elif 'claude' in config.llm_model_name:
            if not config.anthropic_api_key:
                raise UnsupportedModelError("Anthropic API key is required for Claude models")
            self.client = AsyncAnthropic(api_key=config.anthropic_api_key)
            self.provider = 'anthropic'
        else:
            raise UnsupportedModelError(f"Unsupported model: {config.llm_model_name}")

    async def generate(self, prompt: str) -> str:
        for attempt in range(self.config.max_retries):
            try:
                if self.provider == 'openai':
                    return await self._generate_openai(prompt)
                elif self.provider == 'anthropic':
                    return await self._generate_anthropic(prompt)
            except RateLimitError:
                if attempt == self.config.max_retries - 1:
                    raise
                await asyncio.sleep(2 ** attempt)  # Exponential backoff
            except Exception as e:
                logger.error(f"Error in LLM generation: {str(e)}")
                if attempt == self.config.max_retries - 1:
                    raise LLMError(f"Error in LLM generation: {str(e)}")
        raise LLMError("Max retries reached")

    async def _generate_openai(self, prompt: str) -> str:
        try:
            completion = await self.client.chat.completions.create(
                model=self.config.llm_model_name,
                messages=[{"role": "user", "content": prompt}],
                max_tokens=self.config.max_tokens,
                temperature=self.config.temperature
            )
            return completion.choices[0].message.content
        except Exception as e:
            raise LLMError(f"OpenAI API error: {str(e)}")

    async def _generate_anthropic(self, prompt: str) -> str:
        try:
            completion = await self.client.completions.create(
                model=self.config.llm_model_name,
                prompt=f"\n\nHuman: {prompt}\n\nAssistant:",
                max_tokens_to_sample=self.config.max_tokens,
                temperature=self.config.temperature
            )
            return completion.completion
        except Exception as e:
            raise LLMError(f"Anthropic API error: {str(e)}")


from setuptools import setup, find_packages

setup(
    name="medhaai",
    version="0.3.0",
    packages=find_packages(exclude=["tests", "tests.*"]),
    install_requires=[
        "openai>=1.0.0",
        "anthropic>=0.3.0",
        "pydantic>=2.0.0",
        "pydantic-settings>=2.0.0",
        "aiohttp>=3.7.4",
        "beautifulsoup4>=4.9.3",
        "newspaper3k>=0.2.8",
        "nltk>=3.6.2",
        "lxml>=4.6.3",
    ],
)